
!TMessageRouterTests methodsFor: 'router tests' stamp: 'bf 12/20/2018 16:59'!
testBackdoorLatency
	"Test the message roundtrip lateny to the router"
	<timeout: 10>
	| island ref latencyStats avg start label count gcCount |
	"No logging while we're measuring latency"
	log := nil.
	self testMasterStage3.
	island := controller island.
	controller backDoor: router.
	start := Time millisecondClockValue.
	gcCount := Smalltalk vmParameterAt: 7.
	"Run test for five seconds"
	count := 0.
	[Time millisecondClockValue - start < 5000] whileTrue:[
		count := count + 1.
		ref := island future yourself.
		[ref isResolved] whileFalse:[controller processMessages].
	].
	latencyStats := controller latencyStats.
	label := self class name, '>>testBackdoorLatency'.
	Transcript cr; show: '-------------------- ', label, ' -------------------'.
	Transcript crtab; show: 'Count: ', count.
	Transcript crtab; show: 'Min latency: ', latencyStats min asFloat, ' ms'.
	Transcript crtab; show: 'Max latency: ', latencyStats max asFloat, ' ms'.
	Transcript crtab; show: 'Avg latency: ', (avg := latencyStats average) asFloat, ' ms'.

	gcCount = (Smalltalk vmParameterAt: 7)
		ifFalse:[Transcript crtab; show: '***** FULL GC detected *****'].

	"Average roundtrip latency should be WAY below 5ms for the trivial case"
	self assert: avg < 5.0.
	self assert: count > 1000.! !

!TMessageRouterTests methodsFor: 'router tests' stamp: 'bf 12/20/2018 16:59'!
testReplicateSends
	"Connect another participant to the master island"
	| original ref replica sync |
	<timeout: 10>

	self testParticipantStage2.

	"Replicate from controller to participant"
	original := 'Hello World'.
	ref := controller island future at: #hello put: original.
	ref wait.
	sync := participant island future yourself.
	sync wait.
	participant island send:[:obj| replica := obj at: #hello].
	self assert: replica = original.

	"Replicate from participant to controller"
	original := 'Testing, testing, one, two'.
	ref := participant island future at: #hello put: original.
	ref wait.
	sync := controller island future yourself.
	sync wait.
	controller island send:[:obj| replica := obj at: #hello].
	self assert: replica = original.
! !

!TMessageRouterTests methodsFor: 'router tests' stamp: 'bf 12/20/2018 16:59'!
testRoundtripLatency
	"Test the message roundtrip lateny to the router"
	<timeout: 10>
	| island ref latencyStats avg start label count gcCount |
	"No logging while we're measuring latency"
	log := nil.
	self testMasterStage3.
	island := controller island.
	start := Time millisecondClockValue.
	gcCount := Smalltalk vmParameterAt: 7.
	"Run test for five seconds"
	count := 0.
	[Time millisecondClockValue - start < 5000] whileTrue:[
		count := count + 1.
		ref := island future yourself.
		[ref isResolved] whileFalse:[controller processMessages].
	].
	latencyStats := controller latencyStats.
	label := self class name, '>>testRoundtripLatency'.
	Transcript cr; show: '-------------------- ', label, ' -------------------'.
	Transcript crtab; show: 'Count: ', count.
	Transcript crtab; show: 'Min latency: ', latencyStats min asFloat, ' ms'.
	Transcript crtab; show: 'Max latency: ', latencyStats max asFloat, ' ms'.
	Transcript crtab; show: 'Avg latency: ', (avg := latencyStats average) asFloat, ' ms'.
	"If a full GC hits chances are that this run is not valid"
	gcCount = (Smalltalk vmParameterAt: 7)
		ifFalse:[Transcript crtab; show: '***** FULL GC detected *****'].
	"Average roundtrip latency should be WAY below 5ms for the trivial case"
	self assert: avg < 5.0.
	self assert: count > 1000.! !
